@@ -41,6 +41,7 @@ import getpass
 import logging
 import os
 import platform
+import signal
 import subprocess
 import sys
 import tempfile
@@ -120,11 +121,28 @@ class OMCSessionBase(with_metaclass(abc.ABCMeta, object)):
         self._omc_log_file = None
 
     def __del__(self):
-        self.sendExpression("quit()")
+        try:
+          self.sendExpression("quit()")
+        except:
+          pass
         self._omc_log_file.close()
+        if sys.version_info.major >= 3:
+          self._omc_process.wait(timeout=1.0)
+        else:
+          for i in range(0,100):
+            time.sleep(0.01)
+            if self._omc_process.poll() is not None:
+              break
         # kill self._omc_process process if it is still running/exists
         if self._omc_process.returncode is None:
-            self._omc_process.kill()
+            print("OMC did not exit after being sent the quit() command; killing the process with pid=%s" % str(self._omc_process.pid))
+            if sys.platform=="win32":
+                self._omc_process.kill()
+                self._omc_process.wait()
+            else:
+                os.killpg(os.getpgid(self._omc_process.pid), signal.SIGTERM)
+                self._omc_process.kill()
+                self._omc_process.wait()
 
     def _create_omc_log_file(self, suffix):
         if sys.platform == 'win32':
@@ -143,7 +161,8 @@ class OMCSessionBase(with_metaclass(abc.ABCMeta, object)):
             my_env["PATH"] = omhome_bin + os.pathsep + my_env["PATH"]
             self._omc_process = subprocess.Popen(self._omc_command, shell=True, stdout=self._omc_log_file, stderr=self._omc_log_file, env=my_env)
         else:
-            self._omc_process = subprocess.Popen(self._omc_command, shell=True, stdout=self._omc_log_file, stderr=self._omc_log_file)
+            # Because we spawned a shell, and we need to be able to kill OMC, create a new process group for this
+            self._omc_process = subprocess.Popen(self._omc_command, shell=True, stdout=self._omc_log_file, stderr=self._omc_log_file, preexec_fn=os.setsid)
         return self._omc_process
 
     def _set_omc_command(self, omc_path, args):
@@ -163,7 +182,7 @@ class OMCSessionBase(with_metaclass(abc.ABCMeta, object)):
             raise
 
     @abc.abstractmethod
-    def _connect_to_omc(self):
+    def _connect_to_omc(self, timeout):
         pass
 
     # FIXME: we should have one function which interacts with OMC. Either execute OR sendExpression.
@@ -210,7 +229,7 @@ class OMCSessionBase(with_metaclass(abc.ABCMeta, object)):
             if parsed:
                 res = self.execute(expression)
             else:
-                res = self._omc.sendExpression(expression)
+                res = self.sendExpression(expression, parsed=False)
         except Exception as e:
             logger.error("OMC failed: {0}, {1}, parsed={2}".format(question, opt, parsed))
             raise e
@@ -313,7 +332,7 @@ class OMCSessionBase(with_metaclass(abc.ABCMeta, object)):
         try:
             return self.ask('getParameterNames', className)
         except KeyError as ex:
-            logger.warning('OMPython error: {0}'.format(ex.message))
+            logger.warning('OMPython error: {0}'.format(ex))
             # FIXME: OMC returns with a different structure for empty parameter set
             return []
 
@@ -339,7 +358,7 @@ class OMCSessionBase(with_metaclass(abc.ABCMeta, object)):
                 OMParser.result = {}
                 return answer[2:]
             except (TypeError, UnboundLocalError) as ex:
-                logger.warning('OMParser error: {0}'.format(ex.message))
+                logger.warning('OMParser error: {0}'.format(ex))
                 return result
 
     def getExtendsModifierNames(self, className, componentName):
@@ -357,7 +376,7 @@ class OMCSessionBase(with_metaclass(abc.ABCMeta, object)):
                 OMParser.result = {}
                 return answer[2:]
             except (TypeError, UnboundLocalError) as ex:
-                logger.warning('OMParser error: {0}'.format(ex.message))
+                logger.warning('OMParser error: {0}'.format(ex))
                 return result
 
     def getNthComponentModification(self, className, comp_id):
@@ -396,7 +415,7 @@ class OMCSessionBase(with_metaclass(abc.ABCMeta, object)):
 
 class OMCSession(OMCSessionBase):
 
-    def __init__(self, readonly=False):
+    def __init__(self, readonly=False, timeout = 0.25):
         OMCSessionBase.__init__(self, readonly)
         self._create_omc_log_file("objid")
         # set omc executable path and args
@@ -404,12 +423,12 @@ class OMCSession(OMCSessionBase):
         # start up omc executable, which is waiting for the CORBA connection
         self._start_omc_process()
         # connect to the running omc instance using CORBA
-        self._connect_to_omc()
+        self._connect_to_omc(timeout)
 
     def __del__(self):
         OMCSessionBase.__del__(self)
 
-    def _connect_to_omc(self):
+    def _connect_to_omc(self, timeout):
         # add OPENMODELICAHOME\lib\python to PYTHONPATH so python can load omniORB imports
         sys.path.append(os.path.join(self.omhome, 'lib', 'python'))
         # import the skeletons for the global module
@@ -429,7 +448,7 @@ class OMCSession(OMCSessionBase):
             attempts = 0
             while True:
                 if not os.path.isfile(self._ior_file):
-                    time.sleep(0.25)
+                    time.sleep(timeout)
                     attempts += 1
                     if attempts == 10:
                         name = self._omc_log_file.name
@@ -491,7 +510,7 @@ class OMCSession(OMCSessionBase):
 
 class OMCSessionZMQ(OMCSessionBase):
 
-    def __init__(self, readonly=False):
+    def __init__(self, readonly=False, timeout = 0.25):
         OMCSessionBase.__init__(self, readonly)
         self._create_omc_log_file("port")
         # set omc executable path and args
@@ -499,12 +518,12 @@ class OMCSessionZMQ(OMCSessionBase):
         # start up omc executable, which is waiting for the CORBA connection
         self._start_omc_process()
         # connect to the running omc instance using CORBA
-        self._connect_to_omc()
+        self._connect_to_omc(timeout)
 
     def __del__(self):
         OMCSessionBase.__del__(self)
 
-    def _connect_to_omc(self):
+    def _connect_to_omc(self, timeout):
         # Locating and using the IOR
         if sys.platform == 'win32':
             self._port_file = "openmodelica.port." + self._random_string
@@ -513,24 +532,21 @@ class OMCSessionZMQ(OMCSessionBase):
         self._port_file = os.path.join(self._temp_dir, self._port_file).replace("\\", "/")
         self._omc_zeromq_uri = "file:///" + self._port_file
         # See if the omc server is running
-        if os.path.isfile(self._port_file):
-            logger.info("OMC Server is up and running at {0}".format(self._omc_zeromq_uri))
-        else:
-            attempts = 0
-            while True:
-                if not os.path.isfile(self._port_file):
-                    time.sleep(0.25)
-                    attempts += 1
-                    if attempts == 10:
-                        name = self._omc_log_file.name
-                        self._omc_log_file.close()
-                        logger.error("OMC Server is down. Please start it! Log-file says:\n%s" % open(name).read())
-                        raise Exception
-                    else:
-                        continue
+        attempts = 0
+        while True:
+            if not os.path.isfile(self._port_file):
+                time.sleep(timeout)
+                attempts += 1
+                if attempts == 10:
+                    name = self._omc_log_file.name
+                    self._omc_log_file.close()
+                    logger.error("OMC Server is down. Please start it! Log-file says:\n%s" % open(name).read())
+                    raise Exception("OMC Server is down. Could not open file %s" % self._port_file)
                 else:
-                    logger.info("OMC Server is up and running at {0}".format(self._omc_zeromq_uri))
-                    break
+                    continue
+            else:
+                logger.info("OMC Server is up and running at {0} pid={1}".format(self._omc_zeromq_uri, self._omc_process.pid))
+                break
 
         # Read the port file
         with open(self._port_file, 'r') as f_p:
@@ -540,38 +556,39 @@ class OMCSessionZMQ(OMCSessionBase):
         import zmq
         context = zmq.Context.instance()
         self._omc = context.socket(zmq.REQ)
+        self._omc.setsockopt(zmq.LINGER, 0) # Dismisses pending messages if closed
         self._omc.connect(self._port)
 
     def execute(self, command):
         if self._omc is not None:
             self._omc.send_string(command)
-            result = self._omc.recv_string()
             if command == "quit()":
                 self._omc.close()
                 self._omc = None
-                return result
+                return None
             else:
+                result = self._omc.recv_string()
                 answer = OMParser.check_for_values(result)
                 return answer
         else:
-            return "No connection with OMC. Create an instance of OMCSessionZMQ."
+            raise Exception("No connection with OMC. Create an instance of OMCSessionZMQ.")
 
     def sendExpression(self, command, parsed=True):
         if self._omc is not None:
             self._omc.send_string(str(command))
-            result = self._omc.recv_string()
             if command == "quit()":
                 self._omc.close()
                 self._omc = None
-                return result
+                return None
             else:
+                result = self._omc.recv_string()
                 if parsed is True:
                     answer = OMTypedParser.parseString(result)
                     return answer
                 else:
                     return result
         else:
-            return "No connection with OMC. Create an instance of OMCSessionZMQ."
+            raise Exception("No connection with OMC. Create an instance of OMCSessionZMQ.")
 
 # author = Sudeep Bajracharya
 # sudba156@student.liu.se
@@ -595,7 +612,7 @@ class Quantity(object):
 
 
 class ModelicaSystem(object):
-    def __init__(self, fileName=None, modelName=None, lmodel=None, useCorba=False):  # 1
+    def __init__(self, fileName=None, modelName=None, lmodel=[], useCorba=False):  # 1
         """
         "constructor"
         It initializes to load file and build a model, generating object, exe, xml, mat, and json files. etc. It can be called :
@@ -606,7 +623,7 @@ class ModelicaSystem(object):
         ex: myModel = ModelicaSystem("ModelicaModel.mo", "modelName")
         """
 
-        if fileName is None and modelName is None and lmodel is None:  # all None
+        if fileName is None and modelName is None and not lmodel:  # all None
             if useCorba:
                 self.getconn = OMCSession()
             else:
@@ -697,13 +714,13 @@ class ModelicaSystem(object):
                 return
 
         # load Modelica standard libraries if needed
-        if lmodel is not None:
-            loadmodelError = ''
-            loadModelResult = self.requestApi("loadModel", lmodel)
-            loadmodelError = self.requestApi('getErrorString')
-            if loadmodelError:
-                print(loadmodelError)
-                return
+        for element in lmodel:
+            if element is not None:
+                loadmodelError = ''
+                loadModelResult = self.requestApi("loadModel", element)
+                loadmodelError = self.requestApi('getErrorString')
+                if loadmodelError:
+                    print(loadmodelError)
 
         # build model
         # buildModelError = ''
@@ -1248,8 +1265,7 @@ class ModelicaSystem(object):
                 resultfilename = self.modelName + '_res.mat'
                 return
             else:
-                print("Error: application file not generated yet")
-                return
+                raise Exception("Error: application file not generated yet")
         else:
             if (platform.system() == "Windows"):
                 getExeFile = os.path.join(os.getcwd(), '{}.{}'.format(self.modelName, "exe")).replace("\\", "/")
@@ -1275,7 +1291,7 @@ class ModelicaSystem(object):
                 resultfilename = self.modelName + '_res.mat'
                 return
             else:
-                print("Error: application file not generated yet")
+                raise Exception("Error: application file not generated yet")
 
     # to extract simulation results
     def getSolutions(self, *varList):  # 12
